@@ -124,6 +124,14 @@ Standard errors are not available unless you fix it.")
   
   if(any(is.na(m))) 
     stop("'start' specifies unknown arguments")
+
+  ##We use the parscale option to help the optimizer
+  ##We do not overwrite user config
+  if (is.null(control$parscale)){
+    parscale <- abs(unlist(start))
+    parscale[parscale == 0] <- 1
+    control$parscale <- parscale
+  }
   
   formals(nplk) <- c(f[m], f[-m])
   nllh <- function(p, ...) nplk(p, ...)
@@ -394,8 +402,8 @@ schlatherform <- function(data, coord, cov.mod, loc.form, scale.form, shape.form
 
   if (missing(start)) {
 
-    start <- .start.schlather(data, coord, cov.mod, loc.model, scale.model,
-                              shape.model, method = method, ...)
+    start <- .start.schlather(data, coord, covariables, cov.mod, loc.form,
+                              scale.form, shape.form, method = method, ...)
     
     start <- start[!(param %in% names(list(...)))]
   
@@ -415,6 +423,14 @@ schlatherform <- function(data, coord, cov.mod, loc.form, scale.form, shape.form
   
   if(any(is.na(m))) 
     stop("'start' specifies unknown arguments")
+
+  ##We use the parscale option to help the optimizer
+  ##We do not overwrite user config
+  if (is.null(control$parscale)){
+    parscale <- abs(unlist(start))
+    parscale[parscale == 0] <- 1
+    control$parscale <- parscale
+  }
   
   formals(nplk) <- c(f[m], f[-m])
   nllh <- function(p, ...) nplk(p, ...)
