@@ -20,7 +20,7 @@ void smithfull(double *data, double *distVec, int *nSite,
   //Some preliminary steps: Valid points?
   if (*fitmarge){
     for (i=0;i<*nSite;i++){
-      if ((scales[i] <= 0) | (shapes[i] <= -1)){
+      if ((scales[i] <= 0) || (shapes[i] <= -1)){
 	*dns = MINF;
 	return;
       }
